<?php
namespace Teknisa\SUP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\SUP\Modules\Cadastros\Services\Cmp02600ParticularidadesService;

class Cmp02600ParticularidadesController extends \Zeedhi\Framework\Controller\Report
{
    const MESSAGES = [
        '1' => "Particularidade por marca obrigatória já cadastrada!",
        '2' => "Particularidade por marca preferencial já cadastrada!",
        '3' => "Particularidade por restrição à marca já cadastrada!",
        '4' => "Particularidade por fornecedor obrigatório já cadastrada!",
        '5' => "Particularidade por fornecedor preferencial já cadastrada!",
        '6' => "Particularidade por restrição ao fornecedor já cadastrada!",
        '7' => "Particularidade por marca/fornecedor obrigatório já cadastrada!",
        '8' => "Particularidade por marca/fornecedor preferencial já cadastrada!",
        '9' => "Particularidade por restrição à marca/fornecedor já cadastrada!",
    ];
    
    protected $connection;
    protected $newCode;
    protected $environment;
    protected $qr2Strategy;
     protected $stNomeRel;
    protected $stExtensao;
    
    public function __construct (
        reportService $reportService, 
        Cmp02600ParticularidadesService $service, 
        $connection, 
        $newCode,
        $environment, 
        $qr2Strategy
    ) {
        parent::__construct($reportService);
        $this->service     = $service;
        $this->connection  = $connection;
        $this->newCode     = $newCode;
        $this->environment = $environment;
        $this->qr2Strategy = $qr2Strategy;
    }
    
    public function generateCode($newCode)
    {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab(), 1, 1, false);
    }
    
    public function addNewProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $particularidade  = $row['CDTIPPART'];
            $newProduct = true;
            
            for ($i = 0; $i < count($particularidade); $i++) {
                $secondRow['CDPRODUTO'] = $row['CDPRODUTO'];
                $secondRow['CDFILIAL'] = $row['CDFILIAL'];
                $secondRow['NRSEQPART'] = self::generateCode('PARTFILI');
                $secondRow['CDTIPPART'] = $particularidade[$i];
                $secondRow['CDMARCA'] = $row['CDMARCA'];
                $secondRow['CDFORNECED'] = $row['CDFORNECED'];
                
                if($particularidade[$i] <= 3){
                    $secondRow['CDFORNECED'] = null;
                } else if($particularidade[$i] <= 6){
                    $secondRow['CDMARCA'] = null;
                }
                if($newProduct == true){
                    $newProduct = false;
                    self::validaInclusao($secondRow, $response);
                } else {
                    if($particularidade[$i] <= 3){
                        self::validateParticularity($this->service->getExistingBrandParticularities($secondRow));
                        self::brandValidate($secondRow);
                    } else if($particularidade[$i] <= 6){
                        self::validateParticularity($this->service->getExistingProviderParticularities($secondRow));
                        self::providerValidate($secondRow);
                    } else {
                        self::validateParticularity($this->service->getExistingBrandProviderParticularities($secondRow));
                        self::brandProviderValidate($secondRow);
                    }
                }
                $this->service->insertNewProductOrParticularity($secondRow);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaInclusao($row, $response)
    {
        if (!empty($this->service->validaProdutoExiste($row))) {
            throw new \Exception("Já existe particularidade cadastrada para o produto informado!");
        } elseif ($row['CDMARCA'] && empty($this->service->validaMarcaProduto($row))) {
            throw new \Exception("Marca não cadastrada para o produto informado!");
        }
        
        return true;
    }
    
    public function deleteProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $products = $this->service->getProductInformation($row);
            
            foreach ($products as $product) {
                $this->service->deleteProductOrParticularity($product);
            }
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function deleteProductOrParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $this->service->deleteProductOrParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function addNewParticularidadePorMarca(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            self::validateParticularity($this->service->getExistingBrandParticularities($row));
            
            self::brandValidate($row);
            
            $row['NRSEQPART']  = self::generateCode('PARTFILI');
            $row['CDFORNECED'] = null;
            
            $this->service->insertNewProductOrParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function addNewParticularidadePorFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            self::validateParticularity($this->service->getExistingProviderParticularities($row));
            
            self::providerValidate($row);
            
            $row['NRSEQPART']  = self::generateCode('PARTFILI');
            $row['CDMARCA'] = null;
            
            $this->service->insertNewProductOrParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function addNewParticularidadePorMarcaFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            self::validateParticularity($this->service->getExistingBrandProviderParticularities($row));
            
            self::brandProviderValidate($row);
            
            $row['NRSEQPART']  = self::generateCode('PARTFILI');
            
            $this->service->insertNewProductOrParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validateParticularity($particularities)
    {
        if (!empty($particularities)) {
            $cdParticularity = $particularities[0]['CDTIPPART'];
            
            throw new \Exception(self::MESSAGES[$cdParticularity]);
        }
    }
    
    public function brandValidate($row)
    {
        
        if (empty($this->service->validaMarcaProduto($row))) {
            throw new \Exception("Marca não cadastrada para o produto");
        }
        
        if (!empty($this->service->validateBrandExists($row))) {
            throw new \Exception("Marca já cadastrada");
        }
    }
    
    public function providerValidate($row){
        
        if (!empty($this->service->validateProviderExists($row))) {
            throw new \Exception("Fornecedor já cadastrado");
        }
    }
    
    public function brandProviderValidate($row){
        
        if (!empty($this->service->validateBrandProviderExists($row))) {
            throw new \Exception("Fornecedor e Marca já cadastrados");
        }
    }
    
    public function updateBrandParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            if ($row['CDMARCA'] != $row['OLD_CDMARCA']) {
                self::brandValidate($row);
            }
            
            $row['CDFORNECED'] = null;
            
            $this->service->updateParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function updateProviderParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            if ($row['CDFORNECED'] != $row['OLD_CDFORNECED']) {
                self::providerValidate($row);
            }
            
            $row['CDMARCA'] = null;
            
            $this->service->updateParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function updateBrandProviderParticularity(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            if ($row['CDMARCA'] != $row['OLD_CDMARCA'] || $row['CDFORNECED'] != $row['OLD_CDFORNECED']) {
                self::brandProviderValidate($row);
            }
            
            $this->service->updateParticularity($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function autoDeleteParticularities(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            self::prepareRowToProcedure($row);
            
            $this->service->procedureExcluirPartFili($row);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function prepareRowToProcedure($row)
    {
        $row['CDFORNECED'] = $row['CDFORNECED'] == "T" ? null : $row['CDFORNECED']; 
        $row['CDMARCA']    = $row['CDMARCA']    == "T" ? null : $row['CDMARCA']; 
        $row['CDFILIAL']   = $row['CDFILIAL']   == "T" ? null : $row['CDFILIAL'];
        $row['TPRODUTO']   = $row['TPRODUTO']   != "T" ? " "  : $row['TPRODUTO']; 
    }
    
    public function exportParticulatiries(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $procedureReturn = $this->service->procedureIncluiPartFili($row);
            
            if ($procedureReturn['P_ROWSAFFECT'] == 'N') {
                throw new \Exception("Não há dados!");
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    protected $reportMapping = array ();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);

        $this->reportMapping[$this->stNomeRel] =
        array (
            'strategy' => $row['STRATEGY'],
            'parameterMapping' => 
            array (
                'PRODINI'       => 'P_PRODINI',
                'NMPRODINI'     => 'P_NMPRODINI',
                'NMPRODFIN'     => 'P_NMPRODFIN',
                'PRODFIN'       => 'P_PRODFIN',
                'CDFILIAL'      => 'L_CDFILIAL',
                'CDGRUPFILI'    => 'P_CDGRUPFILI',
                'CDMARCA'       => 'L_CDMARCA',
                'CDFORNECED'    => 'L_CDFORNECED',
                'EXCLUI'        => 'P_EXCLUI',
                'CDPRODINI'     => 'P_CDPRODINI',
                'CDPRODFIN'     => 'P_CDPRODFIN',
                'CDTIPPART'     => 'P_CDTIPPART',
                'CDOPERADOR'    => 'P_CDOPERADOR',
                'PRODPREF'      => 'P_PRODPREF',
            ),
        );
    
    }
}