<?php
namespace Teknisa\SUP\Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class AprovadorService
{
    
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $newCode, $environment)
    {
        $this->connection = $entityManager->getConnection();
        $this->newCode    = $newCode;
        $this->environment = $environment;
    }
    
    public function generateCode($newCode)
    {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab());
    }
    
    /********************
     *      INSERTS     *
     * ******************/
    
    public function saveAprovador($row)
    {    
        try {
            $sql = "INSERT INTO ALCADAGERFILI (NRALCADAGERFILI, NRCONFIGALCADA, CDFILIAL, CDOPERADOR, VRAPROVCMP, IDEXIGJUSTIFICATIVA, DTINCLUSAO) VALUES (:NRALCADAGERFILI, :NRCONFIGALCADA, :CDFILIAL, :CDOPERADOR, :VRAPROVCMP, :IDEXIGJUSTIFICATIVA, SYSDATE)";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRALCADAGERFILI'     => self::generateCode('ALCADAGERFILI'),
                    'NRCONFIGALCADA'      => $row['NRCONFIGALCADA'],
                    'CDFILIAL'            => $row['CDFILIAL'],
                    'CDOPERADOR'          => $row['CDOPERADOR'],
                    'VRAPROVCMP'          => $row['VRAPROVCMP'],
                    'IDEXIGJUSTIFICATIVA' => $row['IDEXIGJUSTIFICATIVA'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao cadastrar Aprovador!');
        }
    }
    
    public function saveAprovadorRelacionado($row)
    {    
        try {
            $sql = "INSERT INTO ALCADAGEROPER (NRALCADAGEROPER, NRCONFIGALCADA, CDFILIAL, CDOPERADOR, CDOPERRELACAP, DTINCLUSAO) VALUES (:NRALCADAGEROPER, :NRCONFIGALCADA, :CDFILIAL, :CDOPERADOR, :CDOPERRELACAP, SYSDATE)";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRALCADAGEROPER' => self::generateCode('ALCADAGEROPER'),
                    'NRCONFIGALCADA'  => $row['NRCONFIGALCADA'],
                    'CDFILIAL'        => $row['CDFILIAL'],
                    'CDOPERADOR'      => $row['CDOPERADOR'],
                    'CDOPERRELACAP'   => $row['CDOPERRELACAP'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao cadastrar Aprovador Relacionado!');
        }
    }
    
    
    /********************
     *      UPDATES     *
     * ******************/
     
     
    public function updateAprovador($row)
    {
        try {
            $sql = "UPDATE ALCADAGERFILI SET VRAPROVCMP = :VRAPROVCMP, IDEXIGJUSTIFICATIVA = :IDEXIGJUSTIFICATIVA, DTULTATU = SYSDATE WHERE NRALCADAGERFILI = :NRALCADAGERFILI AND NRCONFIGALCADA = :NRCONFIGALCADA AND CDFILIAL = :CDFILIAL AND CDOPERADOR = :CDOPERADOR";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRALCADAGERFILI'     => $row['NRALCADAGERFILI'],
                    'NRCONFIGALCADA'      => $row['NRCONFIGALCADA'],
                    'CDFILIAL'            => $row['CDFILIAL'],
                    'CDOPERADOR'          => $row['CDOPERADOR'],
                    'VRAPROVCMP'          => $row['VRAPROVCMP'],
                    'IDEXIGJUSTIFICATIVA' => $row['IDEXIGJUSTIFICATIVA'],
                )    
            );
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao atualizar Aprovador!');
        }
    }
    
    public function resetarValorOrdemAprovacaoTodosAprovadoresDaAlcada($row)
    {
        try {
            $sql = "UPDATE ALCADAGERFILI SET VRAPROVCMP = 0, DTULTATU = SYSDATE WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA'  => $row['NRCONFIGALCADA'],
                )    
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao atualizar Aprovadores!');
        }
    }
    
    
    /********************
     *      DELETES     *
     * ******************/
    
    public function deleteAprovador($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGERFILI WHERE NRALCADAGERFILI = :NRALCADAGERFILI";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRALCADAGERFILI' => $row['NRALCADAGERFILI'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovador!');
        }
    }
    
    public function deleteTodosAprovadoresAlcada($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGERFILI WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovadores!');
        }
    }
    
    public function deleteTodosAprovadoresFilialAlcada($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGERFILI WHERE NRCONFIGALCADA = :NRCONFIGALCADA AND CDFILIAL = :CDFILIAL";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                    'CDFILIAL'       => $row['CDFILIAL'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovadores!');;
        }
    }
    
    public function deleteAprovadorRelacionado($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGEROPER WHERE CDFILIAL = :CDFILIAL AND NRCONFIGALCADA = :NRCONFIGALCADA AND CDOPERADOR = :CDOPERADOR AND CDOPERRELACAP = :CDOPERRELACAP";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'CDFILIAL'       => $row['CDFILIAL'],
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                    'CDOPERADOR'     => $row['CDOPERADOR'],
                    'CDOPERRELACAP'  => $row['CDOPERRELACAP'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovador Relacionado!');
        }
    }
    
    public function deleteTodosAprovadoresRelacionadosAlcada($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGEROPER WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovadores Relacionados!');
        }
    }
    
    public function deleteTodosAprovadoresRelacionadosFilialAlcada($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGEROPER WHERE NRCONFIGALCADA = :NRCONFIGALCADA AND CDFILIAL = :CDFILIAL";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                    'CDFILIAL'       => $row['CDFILIAL'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovadores Relacionados!');;
        }
    }
    
    public function deleteTodosAprovadoresRelacionadosAoAprovador($row)
    {
        try {
            $sql = "DELETE FROM ALCADAGEROPER WHERE CDFILIAL = :CDFILIAL AND NRCONFIGALCADA = :NRCONFIGALCADA AND CDOPERADOR = :CDOPERADOR";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'CDFILIAL'       => $row['CDFILIAL'],
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                    'CDOPERADOR'     => $row['CDOPERADOR'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Aprovador Relacionado!');
        }
    }
    
    /********************
     *       FINDS      *
     * ******************/
    
    public function findAprovador($row)
    {
        try {
            $sql = 'SELECT G.NRALCADAGERFILI, G.NRCONFIGALCADA, G.CDFILIAL, G.CDOPERADOR, O.NMOPERADOR, G.VRAPROVCMP, G.NRORG, G.DTINCLUSAO, G.DTULTATU, G.NRORGINCLUSAO, G.CDOPERINCLUSAO, G.NRORGULTATU, G.CDOPERULTATU, ( SELECT COUNT(*) FROM APROVALCADA WHERE NRCONFIGALCADA = G.NRCONFIGALCADA AND CDOPERADOR = NVL( G.CDOPERADOR, CDOPERADOR ) ) APROVACAO FROM ALCADAGERFILI G JOIN OPERADOR O ON G.CDOPERADOR = O.CDOPERADOR WHERE G.NRCONFIGALCADA = :NRCONFIGALCADA AND G.CDFILIAL = :CDFILIAL AND G.CDOPERADOR = :CDOPERADOR';
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                    'CDFILIAL'       => $row['CDFILIAL'],
                    'CDOPERADOR'     => $row['CDOPERADOR'],
                )
            );
            
            return $std->fetchAll()[0];
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function findAlcada($NRCONFIGALCADA)
    {
        $sql = "SELECT C.*, ( SELECT COUNT(*) FROM APROVALCADA WHERE NRCONFIGALCADA = C.NRCONFIGALCADA AND IDAPROVADO = 'PENDENTE' ) APROVACAO FROM CONFIGALCADA C WHERE C.NRCONFIGALCADA = :NRCONFIGALCADA";
        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'NRCONFIGALCADA' => $NRCONFIGALCADA,
            )
        );
        
        return $std->fetchAll()[0];
    }
    
    public function findGestores($filial)
    {
        $sql = "SELECT CDDIRETORIA AS DIRETORIA, CDGOPERAC AS GERENTE, CDSUPERVISOR AS SUPERVISOR, CDPLANEJADORA AS PLANEJAMENTO FROM FILIAL WHERE CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'CDFILIAL' => $filial,
                'NRORG'    => $this->environment->getNrOrg(),
            )
        );
        
        return $std->fetchAll()[0];
    }
    
}
