<?php
namespace Teknisa\SUP\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Sup02000NumeroDeCotacoesPorCompradorController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $entityManager;
    
    public function __construct($crudService, $parameterBag, $entityManager) {
        $this->crudService   = $crudService;
        $this->parameterBag  = $parameterBag;
        $this->entityManager = $entityManager;
    }
    
    public function fornecedorProdutosCotados(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            if(sizeof($conditions) > 0) {
                $dataSet = $this->crudService->getFilteredDataSet("sup02000_fornecedor_produtos_cotados", $conditions)->getRows();
                $dataSet = $this->formatDataSet($dataSet);
            } else {
                $dataSet = [];
            }
            $response->addDataSet(new DataSet("sup02000_fornecedor_produtos_cotados", $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function precoProdutosNegociados(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            if(sizeof($conditions) > 0) {
                $dataSet = $this->crudService->getFilteredDataSet("sup02000_preco_produtos_negociados", $conditions)->getRows();
                $dataSet = $this->formatDataSet($dataSet);
            } else {
                $dataSet = [];
            }
            $response->addDataSet(new DataSet("sup02000_preco_produtos_negociados", $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function precoProdutosNegociadoComprador(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            if(sizeof($conditions) > 0) {
                $dataSet = $this->crudService->getFilteredDataSet("sup02000_preco_produtos_negociados_comprador", $conditions)->getRows();
                $dataSet = $this->formatDataSet($dataSet);
            } else {
                $dataSet = [];
            }
            $response->addDataSet(new DataSet("sup02000_preco_produtos_negociados_comprador", $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function formatDataSet($dataSet) {
        for($i = 0; $i < sizeof($dataSet); $i++) {
            $dataSet[$i]["PRODUTO"]    = $this->maskProduto($dataSet[$i]["CDARVPROD"])  . " " . $dataSet[$i]["NMPRODUTO"];
            $dataSet[$i]["FORNECEDOR"] = $this->maskCNPJCPF($dataSet[$i]["CDFORNECED"]) . " " . $dataSet[$i]["NMRAZSOCFORN"];
        }
        return $dataSet;
    }
    
    public function maskCNPJCPF($value) {
        if($value) {
            $value = substr($value, 1, strlen($value) - 1);
            $length = strlen($value);
            $str = "";
            if($length == 11) {
                $str = preg_replace("/^(\d{3})(\d{3})(\d{3})(\d{2})/", "$1.$2.$3-$4", $value); 
            } else if($length == 14) {
                $str = preg_replace("/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/", "$1.$2.$3/$4-$5", $value);
            }
            return $str;
        }
    }
    
    public function maskProduto($value) {
        if($value) {
            $str = "";
            $length = strlen($value);
            for($i = 0; $i < $length; $i++) {
                if($i == 1 || $i == 3 || $i == 5 || $i == 8) {
                    $str = $str . "." . $value[$i];
                } else {
                    $str = $str . $value[$i];
                }
            }
            return $str;
        }
    }
}